// nested.h

// Implements a graph that supports ports and nesting.


//////////////////////////////////////////////////////////////
//
//  Copyright 2000   Simple Investments. 
//
//  Released into the public domain 27th June 2002.
//  Please change, sell, claim as your own, whatever you like.
//
//////////////////////////////////////////////////////////////

#ifndef NESTED_GRAPH_H
#define NESTED_GRAPH_H

// only need to disable this warning for the ggcl includes.
#pragma warning(push)
#pragma warning(disable : 4003)

#include <vector>
#include <contrib\hash_map>
#include <contrib\hash_set>

#include <ggcl\dynamic_graph.h>
#include <ggcl\properties.h>
 
// un disable the warning messages
#pragma warning(pop)


using namespace ggcl;
using std::pair;
using std::vector;
using std::make_pair;

struct vert_portlist_tag    {  enum { num = 207 }; };
struct edge_portpair_tag    {  enum { num = 208 }; };
struct port_vgraph_tag      {  enum { num = 209 }; };
struct port_egraph_tag      {  enum { num = 213 }; };
struct port_edge_tag        {  enum { num = 210 }; };
struct port_vertex_tag      {  enum { num = 211 }; };
struct vert_subgraph_tag    {  enum { num = 212 }; };

// #defines just to shorten things for myself. 
#define NESTED_TARGS class DirectedS, class VertexPlugin, class EdgePlugin, class PortPlugin
#define NESTED_ARGS DirectedS, VertexPlugin, EdgePlugin, PortPlugin

template <NESTED_TARGS>
class nested_graph : public dynamic_graph <DirectedS, 
									                         plugin <vert_portlist_tag, vector<dynamic_graph<DirectedS>::vertex_descriptor> /*PortList*/, 
                                                   plugin<vert_subgraph_tag, nested_graph<NESTED_ARGS>*, VertexPlugin> >, 
                                           plugin <edge_portpair_tag, pair<dynamic_graph<DirectedS>::vertex_descriptor/*port_descriptor*/, 
                                                                       dynamic_graph<DirectedS>::vertex_descriptor>, EdgePlugin> >
{
public:
	//! this graph will store a parent graph of the same type.
	typedef nested_graph<NESTED_ARGS> NG;
	
  // these 4 typedefs define the plugin for the ports, it stores the vertex owning the port, the edge connecting the port, 
  // the graph the edge belongs to and the graph the vertex belongs to
  typedef plugin<port_vertex_tag, vertex_descriptor, PortPlugin>                PortVertexPlugin;
  typedef plugin<port_edge_tag, edge_descriptor, PortVertexPlugin>              PortEdgePlugin;
  typedef plugin<port_egraph_tag, NG*, PortEdgePlugin>                          PortEdgeGraphPlugin;
  typedef plugin<port_vgraph_tag, NG*, PortEdgeGraphPlugin>                     FinalPortPlugin;

  //! the list of ports can be stored as a graph, for less work (perhaps a dynamic graph is not the best choice??)
	typedef dynamic_graph<DirectedS, FinalPortPlugin, EdgePlugin>                 PortGraph;

  //! a port descriptor can simply use the vertex_descriptor of the graph.
	typedef PortGraph::vertex_descriptor                                          port_descriptor;
  //! list of ports stored by each vertex
  typedef vector<port_descriptor>                                               PortList;
  //! pair of ports stored by each edge
  typedef pair<port_descriptor, port_descriptor>                                PortPair;
  //! plugin for edges in the graph
  typedef plugin<edge_portpair_tag, PortPair, EdgePlugin>                       FinalEdgePlugin;

  //! plugin for vertices in graph
  typedef plugin<vert_subgraph_tag, NG*, VertexPlugin>                          GraphPlugin;    
  typedef plugin <vert_portlist_tag, PortList, GraphPlugin>                     FinalVertexPlugin;

  //! BaseClass for this class (defined a bit more neatly
  typedef dynamic_graph <DirectedS, FinalVertexPlugin, FinalEdgePlugin>         BaseGraph;

  typedef PortGraph::vertex_iterator                                            port_iterator;
  typedef PortList::iterator                                                    vport_iterator;

private:
public:     // temporarily until i add them as friends
	PortGraph	               *portList;				//!< stores the list of all the ports
	int			                  nestLevel;				//!< the level of nesting of this graph
	NG                       *parentGraph;			//!< parent graph of this graph
	vertex_descriptor	        parentVertex;			//!< parent vertex of this graph

private:
public:
  //! used to construct a child graph which will share the port list.
  inline nested_graph(nested_graph<NESTED_ARGS> &pParent);

public:
  //! default constructor
  inline nested_graph();
  //! default destructor
  inline ~nested_graph();
};

// #define these for now, work out a way to do this better later.
#define port_property_accessor vertex_property_accessor 
#define port_iterator vertex_iterator

#include "nested.hpp"

#endif // NESTED_GRAPH_H
